﻿// ***********************************************************************************
// CameraBackgroundColor - session plugin version
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// Licensed under CC0 - https://creativecommons.org/publicdomain/zero/1.0/

using MVR.FileManagementSecure;
using SimpleJSON;
using UnityEngine;

public class CameraBackgroundColor : MVRScript
{
    private const string SAVE_FOLDER = "Saves\\PluginData\\CameraBackgroundColor";
    private const string SAVE_FILENAME = "default";
    private const string SAVE_EXTENSION = "json";
    private Camera mainCam;
    private Camera screenshotCam;
    private Camera thumbnailCam;
    private Camera windowCam;
    private JSONStorableColor jColor;

    public override void Init()
    {
        mainCam = CameraTarget.centerTarget.targetCamera;
        screenshotCam = SuperController.singleton.hiResScreenshotCamera;
        thumbnailCam = SuperController.singleton.screenshotCamera;
        Atom windowCamAtom = GetSceneAtoms().Find((Atom a) => { return a.type == "WindowCamera"; });
        if (windowCamAtom != null)
        {
            CameraControl cameraControl = windowCamAtom.GetStorableByID("CameraControl") as CameraControl;
            if (cameraControl != null)
                windowCam = cameraControl.cameraToControl;
        }

        jColor = new JSONStorableColor("Color", HSVColorPicker.RGBToHSV(0f, 0f, 1f), SetColor);
        //RegisterColor(jColor);
        CreateColorPicker(jColor);

        CreateButton("Save as my default").button.onClick.AddListener(SavePluginDataJSON);
        CreateButton("Delete my default").button.onClick.AddListener(DeletePluginDataJSON);

        // RIGHT SIDE
        UIDynamicTextField uiText = CreateTextField(new JSONStorableString("Description", "To auto-load this with VAM:\n1) Load it as [Session Plugin]\n2) go to [Session Plugin Presets]\n > [Change User Defaults...]\n > [Set Current As User Defaults]\n\n\nCreated by Sally Whitemane\npatreon.com/sallyw\nLicensed under CC0"),true);
        uiText.height = 390f;
    }

    void Start()
    {
        LoadPluginDataJSON();
        SetColor(jColor.val.H, jColor.val.S, jColor.val.V);
    }

    void OnDestroy() { SetColor(0f, 0f, 0f); }

    private void OnEnable() { SetColor(jColor.val.H, jColor.val.S, jColor.val.V); }
    private void OnDisable() { SetColor(0f, 0f, 0f); }
    private void SetColor(float h, float s, float v)
    {
        mainCam.backgroundColor = Color.HSVToRGB(h, s, v);
        screenshotCam.backgroundColor = mainCam.backgroundColor;
        thumbnailCam.backgroundColor = mainCam.backgroundColor;
        windowCam.backgroundColor = mainCam.backgroundColor;
    }

    private void SavePluginDataJSON()
    {
        FileManagerSecure.CreateDirectory(SAVE_FOLDER);
        JSONClass jc = new JSONClass
            {
                {"hue", jColor.val.H.ToString()},
                {"sat", jColor.val.S.ToString()},
                {"val", jColor.val.V.ToString()},
            };

        string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
        this.SaveJSON(jc, path);
        SuperController.LogMessage("CameraBackgroundColor saved " + path);
    }

    private bool LoadPluginDataJSON()
    {
        string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
        if (!FileManagerSecure.FileExists(path))
            // silent fail if user has no default file saved
            return false;
        JSONClass jc = this.LoadJSON(path).AsObject;
        if (jc == null)
        {
            SuperController.LogMessage("CameraBackgroundColor failed to load user default - LoadJSON() == null");
            return false;
        }
        if (!jc.HasKey("hue") || !jc.HasKey("sat") || !jc.HasKey("val"))
        {
            SuperController.LogMessage("CameraBackgroundColor failed to load user default - missing key(s), file modified?");
            return false;
        }
        jColor.SetVal(jc["hue"].AsFloat, jc["sat"].AsFloat, jc["val"].AsFloat);
        return true;
    }

    private void DeletePluginDataJSON()
    {
        string path = SAVE_FOLDER + "\\" + SAVE_FILENAME + "." + SAVE_EXTENSION;
        FileManagerSecure.DeleteFile(path);
        SuperController.LogMessage("CameraBackgroundColor deleted " + path);
    }
}